from pathlib import Path

import kubify.src.core.app_constants as app_constants
from pytest_kind import KindCluster


class Up:
    def __init__(self):
        pass

    def up(self):
        deployment_manifest_yaml = f"""
kind: Cluster
apiVersion: kind.x-k8s.io/v1alpha4
nodes:
- role: control-plane
  extraMounts:
  - hostPath: "{app_constants.root_dir_full_path}"
    containerPath: /var/folders/kubify
    readOnly: true
  kubeadmConfigPatches:
  - |
    kind: InitConfiguration
    nodeRegistration:
      kubeletExtraArgs:
        node-labels: "ingress-ready=true"
  extraPortMappings:
  - containerPort: 80
    hostPort: 8200
    protocol: TCP
  - containerPort: 443
    hostPort: 8443
    protocol: TCP
"""
        kind_yaml = f"{app_constants.kubify_work}/kind.yaml"
        kind_yaml_path = Path(kind_yaml)
        kind_yaml_path.touch(exist_ok=True)
        with open(kind_yaml_path, "w+") as file:
            file.write(deployment_manifest_yaml)
        kind_cluster = KindCluster(name="kubify")
        kind_cluster.create(config_file=kind_yaml)
        kind_cluster.kubectl(
            "apply",
            "-f",
            f"{app_constants.git_dir}/../kubify/ops/templates/k8s/bootstrap.yaml",
        )

    def main(self):
        self.up()


if __name__ == "__main__":
    up = Up()
    up.main()



import boto3
import botocore


class S3Utils:
    def __init__(self):
        self.client = boto3.resource("s3")

    def get_bucket(self, bucket_name):
        try:

            self.client.meta.client.head_bucket(Bucket=bucket_name)
            print("Bucket Exists!")
            bucket = self.client.Bucket(bucket_name)
            return bucket
        except botocore.exceptions.ClientError as e:
            # If a client error is thrown, then check that it was a 404 error.
            # If it was a 404 error, then the bucket does not exist.
            error_code = int(e.response["Error"]["Code"])
            if error_code == 403:
                print("Private Bucket. Forbidden Access!")
                return None
            elif error_code == 404:
                print("Bucket Does Not Exist!")
                return None

    def create_bucket(self, bucket_name, region):
        response = self.client.create_bucket(
            Bucket=bucket_name, CreateBucketConfiguration={"LocationConstraint": region}
        )
        return response

    def put_bucket_encryption(self, bucket_name):
        response = self.client.put_bucket_encryption(
            Bucket=bucket_name,
            ServerSideEncryptionConfiguration={
                "Rules": [
                    {"ApplyServerSideEncryptionByDefault": {"SSEAlgorithm": "AES256"}},
                ]
            },
        )
        return response



"""Console script for Kubify."""
import argparse

# from ast import arg
import sys
import kubify.src.kubify as kubify


def main():
    parser = argparse.ArgumentParser(description="CLI for Kubify")
    #
    parser.add_argument(
        "--up",
        action="store_true",
        default=False,
        help="start kubify kind local rapid testing kubernetes cluster",
    )
    parser.add_argument(
        "--down",
        action="store_true",
        default=False,
        help="pause all running services and local cluster",
    )
    parser.add_argument(
        "--start",
        action="store_true",
        default=False,
        help="start service and it's kubify.yaml depends_on services, listen for changes on all services",
    )
    ##
    parser.add_argument(
        "--test_logger",
        action="store_true",
        default=False,
        help="test logger functionality",
    )
    # parser.add_argument(
    #     "--create_work_dirs", action="store_true", help="in users home kubify directory"
    # )
    # parser.add_argument(
    #     "--set_context_kind_kind",
    #     action="store_true",
    #     help="sets the kuberenetes context to kind",
    # )
    parser.add_argument(
        "--test_or_create_s3_artifacts_bucket",
        action="store_true",
        default=False,
        help="sets the s3 bucket for state file for terraform",
    )
    # parser.add_argument(
    #     "--get_entrypoint", action="store_true", help="gets the entrypoint pod"
    # )
    # parser.add_argument(
    #     "--get_service_pod", action="store_true", help="gets the get_service_pod"
    # )

    if len(sys.argv) <= 1:
        sys.argv.append("--help")

    args = parser.parse_args()

    if args.up:
        kubify.up()

    if args.down:
        kubify.down()

    if args.start:
        kubify.start()

    if args.test_logger:
        kubify.test_logger()
    # if args.create_work_dirs:
    #     kubify.create_work_dirs()
    # if args.set_context_kind_kind:
    #     kubify.set_context_kind_kind()
    if args.test_or_create_s3_artifacts_bucket:
        # this is currently done via the make file
        kubify.test_or_create_s3_artifacts_bucket()
    # if args.get_entrypoint:
    #     kubify.get_entrypoint()
    # if args.get_service_pod:
    #     kubify.get_service_pod()
    # if args.build_entrypoint:
    #     kubify.build_entrypoint()



# from curses import meta
import time
import logging
import operator
import re
from kubernetes import client as k8s_client
from kubernetes import config as k8s_config
from kubernetes.client.rest import ApiException
from .modules.get_cm import K8sConfigMap
import pykube

config = pykube.KubeConfig.from_file()
api = pykube.HTTPClient(pykube.KubeConfig.from_file())
api = pykube.HTTPClient(config)


def api_version():
    logging.info(api.version)


def list_deploy_objects(api):
    list(pykube.Deployment.objects(api))


# Query for all ready pods in a custom namespace:
def query_pods(namespace):
    pykube_api = pykube.HTTPClient(pykube.KubeConfig.from_file())
    pods = pykube.Pod.objects(pykube_api).filter(namespace=namespace)
    ready_pods = filter(operator.attrgetter("ready"), pods)
    return ready_pods


# Access any attribute of the Kubernetes object:
def get_pod_object(namespace, pod_name, pykube_api):
    pod = pykube.Pod.objects(pykube_api).filter(namespace=namespace).get(name=pod_name)
    pod.obj["spec"]["containers"][0]["image"]


def deploy_obj(pykube_api, obj):
    pykube.Deployment(pykube_api, obj).create()


def delete_obj(pykube_api, obj):
    pykube.Deployment(pykube_api, obj).delete()


def get_namespaces():
    # Load the connection config, default behaviour is to load ~/.kube/config
    # For development this is fine, when moving into production another way
    # of providing the config is required
    k8s_config.load_kube_config()

    v1 = k8s_client.CoreV1Api()
    nameSpaceList = v1.list_namespace()
    for nameSpace in nameSpaceList.items:
        logging.info(nameSpace.metadata.name)
    return nameSpaceList


def get_api_resources():
    k8s_config.load_kube_config()
    api_resource_names = []
    for api in k8s_client.ApisApi().get_api_versions().groups:

        versions = []
        for v in api.versions:
            name = ""
            if v.version == api.preferred_version.version and len(api.versions) > 1:
                name += "*"
            name += v.version
            versions.append(name)
        api_name_version = f"{api.name},".join(versions)
        logging.info(api_name_version)
        api_resource_names.append(api_name_version)
    return api_resource_names


def create_namespaced_deployment(
    deployment_name, deployment_manifest, namspace="default"
):
    api_client = k8s_client.ApiClient(config)
    v1 = k8s_client.AppsV1Api(api_client)
    response = v1.create_namespaced_deployment(
        body=deployment_manifest, namespace=namspace
    )
    while True:
        try:
            response = v1.read_namespaced_deployment_status(
                name=deployment_name, namespace=namspace
            )
            if response.status.available_replicas != 3:
                logging.info("Waiting for Deployment to become ready...")
                time.sleep(5)
            else:
                break
        except ApiException as e:
            logging.error(
                f"Exception when calling AppsV1Api -> read_namespaced_deployment_status: {e}\n"
            )


def get_cluster_id(name="kube-system"):
    metadata = k8s_client.V1ObjectMeta(name=name)
    return metadata.uid
    # cluster_context = k8s_config.kube_config.list_kube_config_contexts()
    # print (cluster_context)


def get_cluster_name():
    #  <class 'kubernetes.client.models.v1_config_map.V1ConfigMap'
    cm = K8sConfigMap.get_cm("kube-system")
    for item in cm.items:
        if "kubeadm-config" in item.metadata.name:
            if "clusterName" in item.data["ClusterConfiguration"]:
                cluster_name = re.search(
                    r"clusterName: ([\s\S]+)controlPlaneEndpoint",
                    item.data["ClusterConfiguration"],
                ).group(1)
                print("\nCluster name: {}".format(cluster_name))



from pyhelm.chartbuilder import ChartBuilder
from pyhelm.tiller import Tiller


def install_chart(
    TILLER_HOST, name, type, location, dry_run=False, namespace="default"
):
    tiller = Tiller(TILLER_HOST)
    chart_json = {"name": name, "source": {"type": type, "location": location}}

    chart = ChartBuilder(chart_json)
    # chart = ChartBuilder({"name": "nginx-ingress", "source": {"type": "repo", "location": "https://kubernetes-charts.storage.googleapis.com"}})
    tiller.install_release(chart.get_helm_chart(), dry_run=dry_run, namespace=namespace)


def init():
    # helm init
    # helm init --output json
    pass


# def deployment():
#     container = Container(
#         name="test-container",
#         image="k8s.gcr.io/echoserver:1.4",
#         env=[EnvVar("test", "test-value")],
#         ports=[ContainerPort(8080)],
#     )

# deployment = Deployment(
#     metadata=ObjectMeta(name="test-deployment", labels={"app": "my_app"}),
#     spec=DeploymentSpec(
#         replicas=1,
#         template=PodTemplateSpec(
#             ObjectMeta(labels={"app": "my_app"}),
#             spec=PodSpec(containers=[container]),
#         ),
#         selector=LabelSelector(match_labels={"app": "my_app"}),
#     ),
# )

# builder = install_chart (api_version="3.2.4", name="test", version="0.1.0", app_version="v1", deployment=[deployment])


# def install_chart(api_version="3.2.4", name="test", version="0.1.0", app_version="v1",[deployment]):
#     builder = ChartBuilder(
#         ChartInfo(
#             api_version=api_version,
#             name=name,
#             version=version,
#             app_version=app_version,
#             dependencies= dependencies, #[
#             #     ChartDependency(
#             #         "grafana",
#             #         "5.5.2",
#             #         "https://charts.helm.sh/stable",
#             #         "stable",
#             #         values={"resources": {"requests": {"memory": "100Mi"}}},
#             #     ),
#             #     ChartDependency(
#             #         "local-chart",
#             #         "0.1.0",
#             #         "file:///path/to/my/local-chart",
#             #         "local-repo",
#             #         is_local=True,
#             #     ),
#             # ],
#         ),
#         [],
#     )
#     builder.install_chart({"dependency-update": None})

# from pyhelm.chartbuilder import ChartBuilder
# from pyhelm.tiller import Tiller

# from pyhelm.chartbuilder import ChartBuilder
# from pyhelm.tiller import Tiller


# def install_chart(name="nginx-ingress", dry_run=False):
#     chart = ChartBuilder({'name': 'mongodb', 'source': {'type': 'directory', 'location': '/tmp/pyhelm-kibwtj8d/mongodb'}})
#     t.install_release(chart.get_helm_chart(), dry_run=False, namespace='default')
#     # tiller = Tiller(TILLER_HOST)
#     # chart = ChartBuilder({"name": name, "source": {"type": "repo", "location": "https://kubernetes-charts.storage.googleapis.com"}})
#     # tiller.install_release(chart.get_helm_chart(), dry_run=dry_run, namespace='default')



"""Main module."""

import logging

from logging import config  # noqa

import kubify.src.core.app_constants as app_constants

# TODO fix filename in file handler
log_path = f"{app_constants.log_path}/kubify.log"
log_config = {
    "version": 1,
    "root": {"handlers": ["console", "file"], "level": "DEBUG"},
    "handlers": {
        "console": {
            "formatter": "std_out",
            "class": "logging.StreamHandler",
            "level": "DEBUG",
        },
        "file": {
            "formatter": "std_out",
            "class": "logging.FileHandler",
            "level": "INFO",
            "filename": "kubify.log",
        },
    },
    "formatters": {
        "std_out": {
            "format": """%(asctime)s : %(levelname)s : %(module)s : %(funcName)s : %(lineno)d
                : (Process Details : (%(process)d, %(processName)s),
                Thread Details : (%(thread)d, %(threadName)s))\nLog
                : %(message)s""",
            "datefmt": "%d-%m-%Y %I:%M:%S",
        }
    },
}

logging.config.dictConfig(log_config)
_logger = logging.getLogger()


def setup_logger():
    """_summary_"""
    _logger.info("start logging")



"""Main module."""

import os

# import sys
import os.path

# import glob
# import boto3
import logging

# from collections import namedtuple
# from pathlib import Path

import kubify.src.aws.s3_utils as kubify_utils

import kubify.up as kubify_up

import kubify.down as kubify_down
import kubify.start as kubify_start

import kubify.src.aws_constants as aws_constants

import kubify.src.core.app_constants as app_constants
import kubify.src.core.log as my_logging


# do this before logging for log file to be in work dir
def create_work_dirs():
    if not os.path.exists(app_constants.kubify_work):
        os.makedirs(app_constants.kubify_work)
    if not os.path.exists(app_constants.certs_path):
        os.makedirs(app_constants.certs_path)


create_work_dirs()

# kubify_utils = k8s_utils.K8SUtils()
# os.environ["K8S_OVERRIDE_CONTEXT"] = "kind-kind"

# KUBIFY_DEBUG = True
# KUBIFY_OUT = "/dev/null"
# ANSIBLE_VERBOSITY = 1


# def read_flag_verbose():
#     if KUBIFY_DEBUG:
#         # set -o xtrace
#         ANSIBLE_VERBOSITY = 4
#         KUBIFY_OUT = "/dev/stdout"
#     else:
#         ANSIBLE_VERBOSITY = 1
#         KUBIFY_OUT = "/dev/null"
#     os.environ["ANSIBLE_VERBOSITY"] = ANSIBLE_VERBOSITY

my_logging.setup_logger()
_logger = logging.getLogger()


def test_logger():
    _logger.info("test logger")
    _logger.debug("test logger")
    _logger.warning("test logger")
    _logger.error("test logger")
    _logger.critical("test logger")


# def kubify_version():
#     return git_utils.git_version()


def up():
    kubify_up_class = kubify_up.Up()
    kubify_up_class.up()


def down():
    kubify_down_class = kubify_down.Down()
    kubify_down_class.down()


def start():  # start service in rapid testing coding mode
    kubify_start_class = kubify_start.Start()
    kubify_start_class.start()


# def generate_certs():
#     path = Path(f"{app_constants.kubify_work}/certs/ca.key")
#     if not path.is_file():
#         logging.info("generating ca.key")
#         # TODO fix this
#         certs.create_signed_cert(cn="www.kubify.com")


# def build_image(image_name, src_path):
#     client = docker.from_env()
#     client.images.build(
#         path=src_path,
#         tag=f"{image_name}:latest",
#     )


# def get_service_pod(APP_NAME):
#     # timeout 10 ${KUBECTL_NS} rollout status -w deployment/${APP_NAME} &> /dev/null
#     # echo $(${KUBECTL_NS} get pods -o wide --field-selector=status.phase=Running -l app=${APP_NAME} --no-headers | cut -d ' ' -f1 | head -n 1)
#     pass


# # def update_registry_secret():
# # def update_npm_secret():
# # def get_npm_secret_direct
# # def get_npm_secret

# # def generate_local_cluster_cert():
# #     docker run -e COMMON_NAME="*.${KUBIFY_LOCAL_DOMAIN}" -v "${WORK_DIR}/certs:/certs" -w /certs -it alpine:latest sh -c ./gen-certs.sh


# def debug():
#     logging.info("!!ALL THE kube-system NAMESPACE OBJECTS:")
#     kind.get_api_resources()
#     # $KUBECTL api-resources --verbs=list --namespaced -o name | xargs -n 1 $KUBECTL get --show-kind --ignore-not-found -n kube-system
#     logging.info("!!ALL THE kubify NAMESPACE OBJECTS:")
#     # $KUBECTL api-resources --verbs=list --namespaced -o name | xargs -n 1 $KUBECTL get --show-kind --ignore-not-found -n demo


# def configure_cluster():
#     MANIFESTS = f"{app_constants.k8s_path}"
#     TILLERLESS = os.environ.get("TILLERLESS", False)
#     UPSTREAM = os.environ.get("UPSTREAM", False)
#     if TILLERLESS:
#         TILLER = "tiller run helm"
#     else:
#         TILLER = ""
#     logging.info("Configuring cluster")
#     logging.info("Printing Kind K8s Cluster ID..")
#     logging.info("Kubernetes Cluster ID ->")

#     k8s_cluster_id = (
#         kind.get_cluster_id()
#     )  # `kubectl get ns kube-system -o=jsonpath='{.metadata.uid}'`
#     logging.info(f"{k8s_cluster_id}")
#     logging.info("<-")
#     logging.info(
#         f"""
#       Go to https://license-issuer.appscode.com
#       Register for a license for KubeDB Product
#       Choose 'KubeDB Community Edition'
#       Put in the kubernetes cluster ID: {k8s_cluster_id}
#       After receiving license in email from registering
#       Copy the license file to ~/kubify/kubedb.txt
#       IMPORTANT NOTE: DO NOT SKIP THIS STEP (unless you are in-place re-installing on existing kind cluster)
#         BUT WHY: The liscense file ~/kubify/kubedb.txt is unique to each kind (kubernetes) cluster..........
#       Click enter to continue (after placing fle) 😎"
#       """
#     )
#     # TODO
#     # app_code = get_user_input():#read
#     logging.info(
#         "Thank you! 😎 Continuing Kubify installer"
#     )
#     # if [ -z "$PROFILE" ]; then
#     # KUBECTL="kubectl"
#     # HELM="helm"
#     # else
#     # KUBECTL="kubectl --context ${PROFILE}"
#     # HELM="helm --kube-context ${PROFILE}"
#     # fi
#     # $HELM repo add stakater https://stakater.github.io/stakater-charts
#     # $HELM repo add stable   https://charts.helm.sh/stable
#     # $HELM repo add appscode https://charts.appscode.com/stable/
#     # $HELM repo add jetstack https://charts.jetstack.io
#     # # https://www.digitalocean.com/community/tutorials/how-to-set-up-an-nginx-ingress-on-digitalocean-kubernetes-using-helm
#     # $HELM repo add ingress-nginx https://kubernetes.github.io/ingress-nginx
#     # $HELM repo update
#     # kubectl create namespace demo || true
#     # Testing New Version:
#     # helm repo add appscode https://charts.appscode.com/stable/
#     # helm repo update
#     # KUBEDB_VERSION=v2021.12.21
#     # # KUBEDB_VERSION=v0.20.0
#     # KUBEDB_CATALOG_VERSION=v2021.12.21
#     # # must use "demo" namespace in free edition
#     # helm install kubedb appscode/kubedb \
#     #   --version ${KUBEDB_VERSION} \
#     #   --namespace demo --create-namespace \
#     #   --set-file global.license="${HOME}/kubify/kubedb.txt" || true
#     # if `$KUBECTL --namespace demo get pods | grep kubedb` ; then
#     #     echo "KubeDB is already installed, so running upgrade command instead.."
#     #     echo "TODO: remove the '|| true' workaround once they release a stable KubeDB version release (since they already fixed in master looks like)"
#     #     $HELM ${TILLER} upgrade kubedb appscode/kubedb --install --version $KUBEDB_VERSION --namespace demo || true
#     # else
#     #     echo "Installing KubeDB.."
#     #     # #TODO: look into why the uninstaller for the recent release of kubedb is borked, but for now another workaround
#     #     # $KUBECTL delete psp elasticsearch-db || true
#     #     # $KUBECTL delete psp maria-db || true
#     #     # memcached-db
#     #     # mongodb-db
#     #     # mysql-db
#     #     # percona-xtradb-db
#     #     # postgres-db
#     #     # proxysql-db
#     #     # redis-db
#     #     echo "TODO: remove the '|| true' workaround once they release a stable KubeDB version release (since they already fixed in master looks like)"
#     #     $HELM ${TILLER} install kubedb appscode/kubedb --version $KUBEDB_VERSION --namespace demo || true
#     # fi


# # TODO: also needs uninstall ("undeploy") for reset
# def run_ansible(
#     playbooks=["../ops/ansible/install_kubify_on_mac.yaml"],
#     uninstall="no",
#     tags=["aws"],
# ):
#     Options = namedtuple("Options", [])
#     options = Options(verbosity=None, check=False, tags=tags)
#     PlaybookExecutor(
#         playbooks=playbooks,
#         inventory=None,
#         variable_manager=None,
#         loader=None,
#         options=options,
#         passwords=None,
#     ).run()


# def clean_secrets(env, app_name):
#     # TODO add safety check
#     file_list = glob.glob(f"{app_constants.cloud_formation_path}/*")
#     file_utils.delete_file_list(file_list)
#     file_list = glob.glob(f"{app_constants.secrets_path}/secr*")
#     file_utils.delete_file_list(file_list)
#     file_list = glob.glob(f"{app_constants.secrets_path}/gen-*")
#     file_utils.delete_file_list(file_list)
#     file_list = glob.glob(f"{app_constants.secrets_path}/*.log")
#     file_utils.delete_file_list(file_list)


# def service_setup_secrets(env):
#     # TODO double check these paths
#     secrets_file = f"{app_constants.secrets_path}/secrets.{env}.enc.yaml"
#     config_path = os.path.join(app_constants.app_path, "config")
#     config_file = f"{config_path}/config.{env}.enc.yaml"
#     if not os.path.isfile(secrets_file):
#         pass


# def service_start_dependencies():
#     pass  # start_dependencies "${APP_DIR}" read yaml find depency run if not running (might have started with start for code listening) if running skip, follow dependency change start them all


# def cloud_deploy_services():
#     pass


# def print_local_debug_info():
#     pass


# def init():
#     # set context to kind-kind
#     pass


# def service_init():
#     # ansible-playbook \
#     #     --connection=local \
#     #     --inventory=127.0.0.1, "${K8S_DIR}/ansible/service.yaml" \
#     #     --extra-vars="${EXPOSE_SERVICE_FLAG} env_domain=${ENV_DOMAIN} profile=${SERVICE_PROFILE} ${IS_LOCAL} cert_issuer=${CERT_ISSUER} \
#     #         ${KUBIFY_DOMAIN_ENV} kubify_domain_suffix=${KUBIFY_DOMAIN_SUFFIX} build_profile=${BUILD_PROFILE} skaffold_namespace=${NAMESPACE} \
#     #         env=${ENV} kubify_dir=${WORK_DIR} app_dir=${APP_DIR} app_name=${APP_NAME} app_image=${IMAGE} app_cicd_build_image=${CI_BUILD_IMAGE} \
#     #         kubify_version=${KUBIFY_CURRENT_VERSION} ${VARS}" \
#     #     --tags=$TAGS
#     pass
#     # this is the heart of the program
#     # takes all the input from kubify yaml and extracts number of config mainfest db cloudformation files
#     # lambda etc
#     # just call the ansible


# def service(command="start"):
#     pass
#     # if command == "start":  # default
#     #     service_init
#     # read yaml if aws_only or false then go to skaffold


# #   cat "${APP_DIR}/kubify.yml" | grep aws_only | grep true || ${SKAFFOLD} dev \
# #         --cache-artifacts \
# #         --filename ${WORK_DIR}/${ENV}/${APP_NAME}/skaffold.yaml \
# #         --profile $SKAFFOLD_PROFILE \
# #         --no-prune \
# #         --no-prune-children \
# #         --trigger='polling' \
# #         --port-forward=false


# def service_stop():
#     pass


def test_or_create_s3_artifacts_bucket(
    bucket_name=aws_constants.BUCKET_NAME,
    region=aws_constants.AWS_REGION,
    dr_replication=True,
):
    print(
        "checking access to artifacts s3 bucket to exist, creating it (with encryption at rest enabled) if it does not exist.."
    )
    s3 = kubify_utils.s3_utils.S3Utils()
    bucket = s3.get_bucket(bucket_name)
    if bucket:
        print("success: s3 bucket access working")
    else:
        print("could not find s3 bucket, so creating it")
        s3.create_bucket(bucket_name, region)
        s3.put_bucket_encryption(bucket_name)
        # versioning = s3.BucketVersioning(bucket_name)
        # versioning.enable()
        # s3.put_public_access_block(
        #     Bucket=bucket_name,
        #     PublicAccessBlockConfiguration={
        #         "BlockPublicAcls": True,
        #         "IgnorePublicAcls": True,
        #         "BlockPublicPolicy": True,
        #         "RestrictPublicBuckets": True,
        #     },
        # )
        # s3.put_bucket_versioning(
        #     Bucket=bucket_name,
        #     VersioningConfiguration={"Status": "Enabled"},
        # )
        # if dr_replication:
        #     primary_bucket_name = bucket_name
        #     dr_bucket_name = f"{bucket_name}-dr"
        #     dr_region = "us-east-1"  # region with latest features, so it's good for DR workloads
        #     test_or_create_s3_artifacts_bucket(
        #         bucket_name=dr_bucket_name, region=dr_region, dr_replication=False
        #     )
        #     client = boto3.client("sts")
        #     account_id = client.get_caller_identity()["Account"]
        #     s3.put_bucket_replication(
        #         Bucket=primary_bucket_name,
        #         # Modify the entry below with your account and the replication role you created
        #         ReplicationConfiguration={
        #             "Role": f"arn:aws:iam::{account_id}:role/ReplicationRole",
        #             "Rules": [
        #                 {
        #                     "Priority": 1,
        #                     "Destination": {"Bucket": f"arn:aws:s3:::{dr_bucket_name}"},
        #                     "Status": "Enabled",
        #                 },
        #             ],
        #         },
        #     )
        # print("s3 bucket replication, versioning and security set")


# def set_context_kind_kind():
#     kubify_utils.set_context_kind_kind()


# def get_entrypoint():
#     kubify_utils.get_entrypoint()


# def build_entrypoint():
#     kubify_utils.build_entrypoint()


# def get_service_pod(pod_name):
#     kubify_utils.get_service_pod(pod_name)


# if __name__ == "__main__":
#     k8s_utils = k8s_utils.K8SUtils()
#     os.environ["K8S_OVERRIDE_CONTEXT"] = "kind-kind"



import os

from pathlib import Path

env = "dev"
app_name = "kubify"
dir_path = os.path.dirname(os.path.realpath(__file__))
# what is this for?
cwd = os.getcwd()
root_dir = os.path.join(*[dir_path, "..", "..", ".."])
root_dir_full_path = os.path.abspath(root_dir)
git_dir = os.path.join(root_dir_full_path, ".git")
home = str(Path.home())
# WORK_DIR
WORK_DIR = os.path.join(*[home, ".kubify"])
kubify_work = os.path.join(*[home, ".kubify"])
if not os.path.exists(kubify_work):
    os.makedirs(kubify_work)
k8s_path = os.path.join(*[root_dir_full_path, "kubify", "ops", "templates", "k8s"])
log_path = os.path.join(kubify_work, "logs")
certs_path = os.path.join(kubify_work, "certs")
cloud_formation_path = os.path.join(*[kubify_work, env, app_name, "cloudformation"])
manifests_path = os.path.join(*[kubify_work, env, app_name, "manifests"])
app_path = os.path.join(*[kubify_work, env, app_name])
secrets_path = os.path.join(app_path, "secrets")
secrets_file = f"{secrets_path}/secrets.{env}.enc.yaml"
config_path = os.path.join(app_path, "config")
config_file = f"{config_path}/config.{env}.enc.yaml"
#
ops_dir = os.path.join(*[root_dir_full_path, "kubify", "ops"])
ansible_dir = os.path.join(*[ops_dir, "ansible"])


KUBIFY_LOCAL_DOMAIN_SUFFIX = "kubify.local"  # Local domain suffix
KUBIFY_LOCAL_DOMAIN = (
    f"local.{KUBIFY_LOCAL_DOMAIN_SUFFIX}"  # The local domain (for development)
)
KUBIFY_UPSTREAM_DOMAIN_SUFFIX = os.environ.get(
    "KUBIFY_UPSTREAM_DOMAIN_SUFFIX", "kubify.com"
)  # The domain suffix for upstream environments (Example: <env>.kubify.local)
AWS_REGION = os.environ.get("KUBIFY_AWS_REGION", "us-east-1")
AWS_PROFILE = os.environ.get("KUBIFY_AWS_PROFILE", "default")
AWS_ACCOUNT_ID = ""  # TODO

KUBIFY_UPSTREAM_ENV_ACCOUNT = f"arn:aws:eks:{AWS_REGION}:{AWS_ACCOUNT_ID}"
# KUBIFY_NPM_CREDENTIALS_SECRET = "npm-credentials"

USER_NAME = "git config --get user.name"
ALL_ENV = ["dev", "test", "stage", "prod"]
KMS_KEY_NAME = f"kubify_secrets_{env}"
DEV_KMS = f"arn:aws:kms:{AWS_REGION}:{AWS_ACCOUNT_ID}:alias/{KMS_KEY_NAME}"
TEST_KMS = f"arn:aws:kms:{AWS_REGION}:{AWS_ACCOUNT_ID}:alias/{KMS_KEY_NAME}"
STAGE_KMS = f"arn:aws:kms:{AWS_REGION}:{AWS_ACCOUNT_ID}:alias/{KMS_KEY_NAME}"
PROD_KMS = f"arn:aws:kms:{AWS_REGION}:{AWS_ACCOUNT_ID}:alias/{KMS_KEY_NAME}"

# vars
os.environ["KUBECONFIG"] = os.environ.get(
    "KUBIFY_KUBECONFIG", f"{git_dir}/../.pytest-kind/kubify/kubeconfig"
)



from kubernetes import client
from kubernetes.client.rest import ApiException

# from .load_kube_config import kubeConfig

# kubeConfig.load_kube_config()
core = client.CoreV1Api()


class K8sConfigMap:
    def get_cm(ns, logger):
        try:
            if ns != "all":
                logger.info("Fetching {} namespace configMaps data...".format(ns))
                namespace = ns
                configmaps = core.list_namespaced_config_map(
                    namespace, timeout_seconds=10
                )
            else:
                logger.info("Fetching all namespace configMaps data.")
                configmaps = core.list_config_map_for_all_namespaces(timeout_seconds=10)
            return configmaps
        except ApiException as e:
            logger.warning(
                "Exception when calling CoreV1Api->list_namespaced_config_map: %s\n" % e
            )



from pytest_kind import KindCluster


class Down:
    def __init__(self):
        pass

    def down(self):
        kind_cluster = KindCluster(name="kubify")
        kind_cluster.delete()

    def main(self):
        self.down()


if __name__ == "__main__":
    down = Down()
    down.main()



import kubify as module_0


def test_case_0():
    try:
        module_0.test_or_create_s3_artifacts_bucket()
    except BaseException:
        pass


def test_case_1():
    try:
        module_0.test_or_create_s3_artifacts_bucket()
    except BaseException:
        pass


# def test_case_2():
#     try:
#     except BaseException:
#         pass


def test_case_3():
    try:
        str_0 = " zZheE,L)|];Hk>#r."
        float_0 = 820.059182
        int_0 = 10
        bool_0 = False
        tuple_0 = str_0, float_0, int_0, bool_0
        module_0.test_or_create_s3_artifacts_bucket(tuple_0)
    except BaseException:
        pass


def test_case_4():
    try:
        str_0 = "_8cI@9!+~uA:}=hO?;)%"
        module_0.test_or_create_s3_artifacts_bucket(str_0)
    except BaseException:
        pass


def test_case_5():
    try:
        module_0.test_or_create_s3_artifacts_bucket()
    except BaseException:
        pass


def test_case_6():
    try:
        float_0 = 2911.043
        module_0.test_or_create_s3_artifacts_bucket(float_0)
    except BaseException:
        pass


def test_case_7():
    try:
        module_0.test_or_create_s3_artifacts_bucket()
    except BaseException:
        pass



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("USAGE.rst") as usage_file:
    usage = usage_file.read()

requirements = [
    "pytest-kind",
    "kubernetes",
    "boto3",
    "awscli",
    "ansible-runner",
]

test_requirements = [
    "pytest",
    "pytest-cov",
    "tox",
    "pick",
    "bandit",
    "black",
    "flake8",
    "bump2version",
]

extras = {
    "tests": test_requirements,
    "test": test_requirements,
    "develop": [
        "Sphinx",
    ]
    + test_requirements,
}

setup(
    author="kubify",
    author_email="w@kubify.com",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    description="Automated EKS",
    entry_points={
        "console_scripts": [
            "kubify=kubify.cli:main",
        ],
    },
    install_requires=requirements,
    license="BSD license",
    long_description=readme + "\n\n" + usage,
    include_package_data=True,
    keywords="kubify",
    name="kubify",
    packages=find_packages(include=["kubify", "kubify.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    extras_require=extras,
    url="https://github.com/willyguggenheim/kubify",
    version="9013.0.2",
    zip_safe=False,
)



# TODO (UNCOMMENT)
# import os
# import kubify.src.core.app_constants as app_constants
# from collections import namedtuple
# import ansible_runner

# TODO (COMMENT)
import subprocess  # nosec - B404: placeholder until ported to python fully


class Start:
    def __init__(self):
        pass

    def start(self):
        # TODO (COMMENT)
        subprocess.run(["kubify_port.sh", "start"])  # nosec: B603 B607 - placeholder
        # TODO (UNCOMMENT)
        # tags = ([f"common,{app_constants.dir_path}"],)
        # Options = namedtuple("Options", [])
        # options = Options(verbosity=1, check=False, tags=tags)
        # print(os.environ["KUBECONFIG"])
        # ansible_runner.interface.run(
        #     playbook=f"{app_constants.ops_dir}/ansible/service.yaml",
        #     verbosity=1,
        #     options=options,
        #     tags=tags,
        # )

    def main(self):
        self.start()


if __name__ == "__main__":
    start = Start()
    start.main()



import glob
import os
import re
import sys

from binascii import a2b_hex

from tornado.httpclient import AsyncHTTPClient
from kubernetes import client
from jupyterhub.utils import url_path_join

# Make sure that modules placed in the same directory as the jupyterhub config are added to the pythonpath
configuration_directory = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, configuration_directory)

from z2jh import (
    get_config,
    set_config_if_not_none,
    get_name,
    get_name_env,
    get_secret_value,
)


def camel_caseify(s):
    """convert snake_case to camelCase

    For the common case where some_value is set from someValue
    so we don't have to specify the name twice.
    """
    return re.sub(r"_([a-z])", lambda m: m.group(1).upper(), s)


# Configure JupyterHub to use the curl backend for making HTTP requests,
# rather than the pure-python implementations. The default one starts
# being too slow to make a large number of requests to the proxy API
# at the rate required.
AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")

c.JupyterHub.spawner_class = "kubespawner.KubeSpawner"

# Connect to a proxy running in a different pod. Note that *_SERVICE_*
# environment variables are set by Kubernetes for Services
c.ConfigurableHTTPProxy.api_url = (
    f'http://{get_name("proxy-api")}:{get_name_env("proxy-api", "_SERVICE_PORT")}'
)
c.ConfigurableHTTPProxy.should_start = False

# Do not shut down user pods when hub is restarted
c.JupyterHub.cleanup_servers = False

# Check that the proxy has routes appropriately setup
c.JupyterHub.last_activity_interval = 60

# Don't wait at all before redirecting a spawning user to the progress page
c.JupyterHub.tornado_settings = {
    "slow_spawn_timeout": 0,
}


# configure the hub db connection
db_type = get_config("hub.db.type")
if db_type == "sqlite-pvc":
    c.JupyterHub.db_url = "sqlite:///jupyterhub.sqlite"
elif db_type == "sqlite-memory":
    c.JupyterHub.db_url = "sqlite://"
else:
    set_config_if_not_none(c.JupyterHub, "db_url", "hub.db.url")
db_password = get_secret_value("hub.db.password", None)
if db_password is not None:
    if db_type == "mysql":
        os.environ["MYSQL_PWD"] = db_password
    elif db_type == "postgres":
        os.environ["PGPASSWORD"] = db_password
    else:
        print(f"Warning: hub.db.password is ignored for hub.db.type={db_type}")


# c.JupyterHub configuration from Helm chart's configmap
for trait, cfg_key in (
    ("concurrent_spawn_limit", None),
    ("active_server_limit", None),
    ("base_url", None),
    ("allow_named_servers", None),
    ("named_server_limit_per_user", None),
    ("authenticate_prometheus", None),
    ("redirect_to_server", None),
    ("shutdown_on_logout", None),
    ("template_paths", None),
    ("template_vars", None),
):
    if cfg_key is None:
        cfg_key = camel_caseify(trait)
    set_config_if_not_none(c.JupyterHub, trait, "hub." + cfg_key)

# hub_bind_url configures what the JupyterHub process within the hub pod's
# container should listen to.
hub_container_port = 8081
c.JupyterHub.hub_bind_url = f"http://:{hub_container_port}"

# hub_connect_url is the URL for connecting to the hub for use by external
# JupyterHub services such as the proxy. Note that *_SERVICE_* environment
# variables are set by Kubernetes for Services.
c.JupyterHub.hub_connect_url = (
    f'http://{get_name("hub")}:{get_name_env("hub", "_SERVICE_PORT")}'
)

# implement common labels
# this duplicates the jupyterhub.commonLabels helper
common_labels = c.KubeSpawner.common_labels = {}
common_labels["app"] = get_config(
    "nameOverride",
    default=get_config("Chart.Name", "jupyterhub"),
)
common_labels["heritage"] = "jupyterhub"
chart_name = get_config("Chart.Name")
chart_version = get_config("Chart.Version")
if chart_name and chart_version:
    common_labels["chart"] = "{}-{}".format(
        chart_name,
        chart_version.replace("+", "_"),
    )
release = get_config("Release.Name")
if release:
    common_labels["release"] = release

c.KubeSpawner.namespace = os.environ.get("POD_NAMESPACE", "default")

# Max number of consecutive failures before the Hub restarts itself
# requires jupyterhub 0.9.2
set_config_if_not_none(
    c.Spawner,
    "consecutive_failure_limit",
    "hub.consecutiveFailureLimit",
)

for trait, cfg_key in (
    ("pod_name_template", None),
    ("start_timeout", None),
    ("image_pull_policy", "image.pullPolicy"),
    # ('image_pull_secrets', 'image.pullSecrets'), # Managed manually below
    ("events_enabled", "events"),
    ("extra_labels", None),
    ("extra_annotations", None),
    ("uid", None),
    ("fs_gid", None),
    ("service_account", "serviceAccountName"),
    ("storage_extra_labels", "storage.extraLabels"),
    # ("tolerations", "extraTolerations"), # Managed manually below
    ("node_selector", None),
    ("node_affinity_required", "extraNodeAffinity.required"),
    ("node_affinity_preferred", "extraNodeAffinity.preferred"),
    ("pod_affinity_required", "extraPodAffinity.required"),
    ("pod_affinity_preferred", "extraPodAffinity.preferred"),
    ("pod_anti_affinity_required", "extraPodAntiAffinity.required"),
    ("pod_anti_affinity_preferred", "extraPodAntiAffinity.preferred"),
    ("lifecycle_hooks", None),
    ("init_containers", None),
    ("extra_containers", None),
    ("mem_limit", "memory.limit"),
    ("mem_guarantee", "memory.guarantee"),
    ("cpu_limit", "cpu.limit"),
    ("cpu_guarantee", "cpu.guarantee"),
    ("extra_resource_limits", "extraResource.limits"),
    ("extra_resource_guarantees", "extraResource.guarantees"),
    ("environment", "extraEnv"),
    ("profile_list", None),
    ("extra_pod_config", None),
):
    if cfg_key is None:
        cfg_key = camel_caseify(trait)
    set_config_if_not_none(c.KubeSpawner, trait, "singleuser." + cfg_key)

image = get_config("singleuser.image.name")
if image:
    tag = get_config("singleuser.image.tag")
    if tag:
        image = "{}:{}".format(image, tag)

    c.KubeSpawner.image = image

# Combine imagePullSecret.create (single), imagePullSecrets (list), and
# singleuser.image.pullSecrets (list).
image_pull_secrets = []
if get_config("imagePullSecret.automaticReferenceInjection") and get_config(
    "imagePullSecret.create"
):
    image_pull_secrets.append(get_name("image-pull-secret"))
if get_config("imagePullSecrets"):
    image_pull_secrets.extend(get_config("imagePullSecrets"))
if get_config("singleuser.image.pullSecrets"):
    image_pull_secrets.extend(get_config("singleuser.image.pullSecrets"))
if image_pull_secrets:
    c.KubeSpawner.image_pull_secrets = image_pull_secrets

# scheduling:
if get_config("scheduling.userScheduler.enabled"):
    c.KubeSpawner.scheduler_name = get_name("user-scheduler")
if get_config("scheduling.podPriority.enabled"):
    c.KubeSpawner.priority_class_name = get_name("priority")

# add node-purpose affinity
match_node_purpose = get_config("scheduling.userPods.nodeAffinity.matchNodePurpose")
if match_node_purpose:
    node_selector = dict(
        matchExpressions=[
            dict(
                key="hub.jupyter.org/node-purpose",
                operator="In",
                values=["user"],
            )
        ],
    )
    if match_node_purpose == "prefer":
        c.KubeSpawner.node_affinity_preferred.append(
            dict(
                weight=100,
                preference=node_selector,
            ),
        )
    elif match_node_purpose == "require":
        c.KubeSpawner.node_affinity_required.append(node_selector)
    elif match_node_purpose == "ignore":
        pass
    else:
        raise ValueError(
            "Unrecognized value for matchNodePurpose: %r" % match_node_purpose
        )

# Combine the common tolerations for user pods with singleuser tolerations
scheduling_user_pods_tolerations = get_config("scheduling.userPods.tolerations", [])
singleuser_extra_tolerations = get_config("singleuser.extraTolerations", [])
tolerations = scheduling_user_pods_tolerations + singleuser_extra_tolerations
if tolerations:
    c.KubeSpawner.tolerations = tolerations

# Configure dynamically provisioning pvc
storage_type = get_config("singleuser.storage.type")
if storage_type == "dynamic":
    pvc_name_template = get_config("singleuser.storage.dynamic.pvcNameTemplate")
    c.KubeSpawner.pvc_name_template = pvc_name_template
    volume_name_template = get_config("singleuser.storage.dynamic.volumeNameTemplate")
    c.KubeSpawner.storage_pvc_ensure = True
    set_config_if_not_none(
        c.KubeSpawner, "storage_class", "singleuser.storage.dynamic.storageClass"
    )
    set_config_if_not_none(
        c.KubeSpawner,
        "storage_access_modes",
        "singleuser.storage.dynamic.storageAccessModes",
    )
    set_config_if_not_none(
        c.KubeSpawner, "storage_capacity", "singleuser.storage.capacity"
    )

    # Add volumes to singleuser pods
    c.KubeSpawner.volumes = [
        {
            "name": volume_name_template,
            "persistentVolumeClaim": {"claimName": pvc_name_template},
        }
    ]
    c.KubeSpawner.volume_mounts = [
        {
            "mountPath": get_config("singleuser.storage.homeMountPath"),
            "name": volume_name_template,
        }
    ]
elif storage_type == "static":
    pvc_claim_name = get_config("singleuser.storage.static.pvcName")
    c.KubeSpawner.volumes = [
        {"name": "home", "persistentVolumeClaim": {"claimName": pvc_claim_name}}
    ]

    c.KubeSpawner.volume_mounts = [
        {
            "mountPath": get_config("singleuser.storage.homeMountPath"),
            "name": "home",
            "subPath": get_config("singleuser.storage.static.subPath"),
        }
    ]

# Inject singleuser.extraFiles as volumes and volumeMounts with data loaded from
# the dedicated k8s Secret prepared to hold the extraFiles actual content.
extra_files = get_config("singleuser.extraFiles", {})
if extra_files:
    volume = {
        "name": "files",
    }
    items = []
    for file_key, file_details in extra_files.items():
        # Each item is a mapping of a key in the k8s Secret to a path in this
        # abstract volume, the goal is to enable us to set the mode /
        # permissions only though so we don't change the mapping.
        item = {
            "key": file_key,
            "path": file_key,
        }
        if "mode" in file_details:
            item["mode"] = file_details["mode"]
        items.append(item)
    volume["secret"] = {
        "secretName": get_name("singleuser"),
        "items": items,
    }
    c.KubeSpawner.volumes.append(volume)

    volume_mounts = []
    for file_key, file_details in extra_files.items():
        volume_mounts.append(
            {
                "mountPath": file_details["mountPath"],
                "subPath": file_key,
                "name": "files",
            }
        )
    c.KubeSpawner.volume_mounts.extend(volume_mounts)

# Inject extraVolumes / extraVolumeMounts
c.KubeSpawner.volumes.extend(get_config("singleuser.storage.extraVolumes", []))
c.KubeSpawner.volume_mounts.extend(
    get_config("singleuser.storage.extraVolumeMounts", [])
)

c.JupyterHub.services = []

if get_config("cull.enabled", False):
    cull_cmd = ["python3", "-m", "jupyterhub_idle_culler"]
    base_url = c.JupyterHub.get("base_url", "/")
    cull_cmd.append("--url=http://localhost:8081" + url_path_join(base_url, "hub/api"))

    cull_timeout = get_config("cull.timeout")
    if cull_timeout:
        cull_cmd.append("--timeout=%s" % cull_timeout)

    cull_every = get_config("cull.every")
    if cull_every:
        cull_cmd.append("--cull-every=%s" % cull_every)

    cull_concurrency = get_config("cull.concurrency")
    if cull_concurrency:
        cull_cmd.append("--concurrency=%s" % cull_concurrency)

    if get_config("cull.users"):
        cull_cmd.append("--cull-users")

    if get_config("cull.removeNamedServers"):
        cull_cmd.append("--remove-named-servers")

    cull_max_age = get_config("cull.maxAge")
    if cull_max_age:
        cull_cmd.append("--max-age=%s" % cull_max_age)

    c.JupyterHub.services.append(
        {
            "name": "cull-idle",
            "admin": True,
            "command": cull_cmd,
        }
    )

for key, service in get_config("hub.services", {}).items():
    # c.JupyterHub.services is a list of dicts, but
    # hub.services is a dict of dicts to make the config mergable
    service.setdefault("name", key)

    # As the api_token could be exposed in hub.existingSecret, we need to read
    # it it from there or fall back to the chart managed k8s Secret's value.
    service.pop("apiToken", None)
    service["api_token"] = get_secret_value(f"hub.services.{key}.apiToken")

    c.JupyterHub.services.append(service)


set_config_if_not_none(c.Spawner, "cmd", "singleuser.cmd")
set_config_if_not_none(c.Spawner, "default_url", "singleuser.defaultUrl")

cloud_metadata = get_config("singleuser.cloudMetadata", {})

if cloud_metadata.get("blockWithIptables") == True:
    # Use iptables to block access to cloud metadata by default
    network_tools_image_name = get_config("singleuser.networkTools.image.name")
    network_tools_image_tag = get_config("singleuser.networkTools.image.tag")
    ip_block_container = client.V1Container(
        name="block-cloud-metadata",
        image=f"{network_tools_image_name}:{network_tools_image_tag}",
        command=[
            "iptables",
            "-A",
            "OUTPUT",
            "-d",
            cloud_metadata.get("ip", "169.254.169.254"),
            "-j",
            "DROP",
        ],
        security_context=client.V1SecurityContext(
            privileged=True,
            run_as_user=0,
            capabilities=client.V1Capabilities(add=["NET_ADMIN"]),
        ),
    )

    c.KubeSpawner.init_containers.append(ip_block_container)


if get_config("debug.enabled", False):
    c.JupyterHub.log_level = "DEBUG"
    c.Spawner.debug = True

# load /usr/local/etc/jupyterhub/jupyterhub_config.d config files
config_dir = "/usr/local/etc/jupyterhub/jupyterhub_config.d"
if os.path.isdir(config_dir):
    for file_path in sorted(glob.glob(f"{config_dir}/*.py")):
        file_name = os.path.basename(file_path)
        print(f"Loading {config_dir} config: {file_name}")
        with open(file_path) as f:
            file_content = f.read()
        # compiling makes debugging easier: https://stackoverflow.com/a/437857
        exec(compile(source=file_content, filename=file_name, mode="exec"))

# load potentially seeded secrets
#
# NOTE: ConfigurableHTTPProxy.auth_token is set through an environment variable
#       that is set using the chart managed secret.
c.JupyterHub.cookie_secret = get_secret_value("hub.config.JupyterHub.cookie_secret")
# NOTE: CryptKeeper.keys should be a list of strings, but we have encoded as a
#       single string joined with ; in the k8s Secret.
#
c.CryptKeeper.keys = get_secret_value("hub.config.CryptKeeper.keys").split(";")

# load hub.config values, except potentially seeded secrets already loaded
for app, cfg in get_config("hub.config", {}).items():
    if app == "JupyterHub":
        cfg.pop("proxy_auth_token", None)
        cfg.pop("cookie_secret", None)
        cfg.pop("services", None)
    elif app == "ConfigurableHTTPProxy":
        cfg.pop("auth_token", None)
    elif app == "CryptKeeper":
        cfg.pop("keys", None)
    c[app].update(cfg)

# execute hub.extraConfig entries
for key, config_py in sorted(get_config("hub.extraConfig", {}).items()):
    print("Loading extra config: %s" % key)
    exec(config_py)



"""
Utility methods for use in jupyterhub_config.py and dynamic subconfigs.

Methods here can be imported by extraConfig in values.yaml
"""
from collections import Mapping
from functools import lru_cache
import os

import yaml

# memoize so we only load config once
@lru_cache()
def _load_config():
    """Load the Helm chart configuration used to render the Helm templates of
    the chart from a mounted k8s Secret, and merge in values from an optionally
    mounted secret (hub.existingSecret)."""

    cfg = {}
    for source in ("secret/values.yaml", "existing-secret/values.yaml"):
        path = f"/usr/local/etc/jupyterhub/{source}"
        if os.path.exists(path):
            print(f"Loading {path}")
            with open(path) as f:
                values = yaml.safe_load(f)
            cfg = _merge_dictionaries(cfg, values)
        else:
            print(f"No config at {path}")
    return cfg


@lru_cache()
def _get_config_value(key):
    """Load value from the k8s ConfigMap given a key."""

    path = f"/usr/local/etc/jupyterhub/config/{key}"
    if os.path.exists(path):
        with open(path) as f:
            return f.read()
    else:
        raise Exception(f"{path} not found!")


@lru_cache()
def get_secret_value(key, default="never-explicitly-set"):
    """Load value from the user managed k8s Secret or the default k8s Secret
    given a key."""

    for source in ("existing-secret", "secret"):
        path = f"/usr/local/etc/jupyterhub/{source}/{key}"
        if os.path.exists(path):
            with open(path) as f:
                return f.read()
    if default != "never-explicitly-set":
        return default
    raise Exception(f"{key} not found in either k8s Secret!")


def get_name(name):
    """Returns the fullname of a resource given its short name"""
    return _get_config_value(name)


def get_name_env(name, suffix=""):
    """Returns the fullname of a resource given its short name along with a
    suffix, converted to uppercase with dashes replaced with underscores. This
    is useful to reference named services associated environment variables, such
    as PROXY_PUBLIC_SERVICE_PORT."""
    env_key = _get_config_value(name) + suffix
    env_key = env_key.upper().replace("-", "_")
    return os.environ[env_key]


def _merge_dictionaries(a, b):
    """Merge two dictionaries recursively.

    Simplified From https://stackoverflow.com/a/7205107
    """
    merged = a.copy()
    for key in b:
        if key in a:
            if isinstance(a[key], Mapping) and isinstance(b[key], Mapping):
                merged[key] = _merge_dictionaries(a[key], b[key])
            else:
                merged[key] = b[key]
        else:
            merged[key] = b[key]
    return merged


def get_config(key, default=None):
    """
    Find a config item of a given name & return it

    Parses everything as YAML, so lists and dicts are available too

    get_config("a.b.c") returns config['a']['b']['c']
    """
    value = _load_config()
    # resolve path in yaml
    for level in key.split("."):
        if not isinstance(value, dict):
            # a parent is a scalar or null,
            # can't resolve full path
            return default
        if level not in value:
            return default
        else:
            value = value[level]
    return value


def set_config_if_not_none(cparent, name, key):
    """
    Find a config item of a given name, set the corresponding Jupyter
    configuration item if not None
    """
    data = get_config(key)
    if data is not None:
        setattr(cparent, name, data)



import kubify.src.core.app_constants as app_constants
import os.path
from pathlib import Path


def test_dir_paths():
    env = "dev"
    app_name = "kubify"
    path = Path(f"{app_constants.root_dir}/__version__")
    assert path.is_file()  # nosec - ✅ B101: assert used
    # os.path.isfile(fname)
    path = Path(f"{app_constants.root_dir_full_path}/__version__")
    assert path.is_file()  # nosec - ✅ B101: assert used
    path = Path(f"{app_constants.git_dir}/HEAD")
    assert path.is_file()  # nosec - ✅ B101: assert used
    path = Path(f"{app_constants.k8s_path}/kubify.yaml")
    assert path.is_file()  # nosec - ✅ B101: assert used
    path = Path(f"{app_constants.ops_dir}/ansible")
    assert path.exists()  # nosec - ✅ B101: assert used
    path = Path(f"{app_constants.ops_dir}/terraform")
    assert path.exists()  # nosec - ✅ B101: assert used
    path = Path(f"{app_constants.ansible_dir}")
    assert path.exists()  # nosec - ✅ B101: assert used
